<?php

class Shopware_Controllers_Frontend_MicropaymentOFBase extends Enlight_Controller_Action
{
  var $McPay;

  function __construct(Enlight_Controller_Request_Request $request = NULL, Enlight_Controller_Response_Response $response = NULL)
  {
    define('MCPAY_BASE_PATH', dirname(__FILE__).'/../../mcpay/');
    require_once(MCPAY_BASE_PATH.'classes/class.mcpay.php');
    $this->McPay = new \mcpay();

    parent::__construct($request, $response);
  }

  /**
   * getSnippet
   *
   * @param $name
   * @param $default
   *
   * @return mixed
   */
  public function getSnippet($name, $default)
  {
    return Shopware()->Snippets()->getNamespace('frontend/checkout/confirm')->get($name, $default);
  }

  /**
   * getStyleAction
   *
   * @return bool
   */
  public function getStyleAction()
  {
    $this->container->get('front')->Plugins()->ViewRenderer()->setNoRender();

    $this->Response()->clearAllHeaders();
    $this->Response()->setHeader('Content-Type', 'text/css');
    //header('Content-Type: text/css');

    $req = $this->Request()->getParams();
    //echo '<pre>'.print_r($req,1).'</pre>';
    if (empty($req['sn'])) {
      if ($this->McPay->debug) $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->Req empty: '.print_r($req['sn'], 1));
      $this->Response()->setHttpResponseCode(403);
      die('403');
    }

    $filename = MCPAY_BASE_PATH.'css/'.$req['sn'].'.css';
    if (!file_exists($filename)) {
      if ($this->McPay->debug) $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->File not found: '.print_r($filename, 1));
      $this->Response()->setHttpResponseCode(404);
      die('404');
    }

    $content = file_get_contents($filename);

    // icon paths
    if (strpos($filename, 'mcpay_formstyle_') !== FALSE){
      $baseURL = $this->Front()->Router()->assemble(array('forceSecure' => 1, 'action' => 'getImage', 'controller' => 'MicropaymentCcard')).'/sn/';
      $repl = array(
        'url("../img/icon_' => 'url("'.$baseURL.'icon_',
 	      '.png' => '',
	      'user' => 'use',
      );
      $content = strtr($content, $repl);
    }

    echo $content;
  }

  /**
   * getScriptAction
   *
   * @return bool
   */
  public function getScriptAction()
  {
    $this->container->get('front')->Plugins()->ViewRenderer()->setNoRender();

    $this->Response()->clearAllHeaders();
    $this->Response()->setHeader('Content-Type', 'application/javascript');
    //header('Content-Type: application/javascript');

    $req = $this->Request()->getParams();
    if (empty($req['sn'])) {
      if ($this->debug) $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->Req empty: '.print_r($req['sn'], 1));
      $this->Response()->setHttpResponseCode(403);
      die('403');
    }

    $filename = MCPAY_BASE_PATH.'js/'.$req['sn'].'.js';
    if (!file_exists($filename)) {
      if ($this->debug) $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->File not found: '.print_r($filename, 1));
      $this->Response()->setHttpResponseCode(404);
      die('404');
    }

    $config = $this->getCfg();
    if (empty($config['payformid'])) {
      $config['payformid'] = 'confirm--form';
    }
    //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$config', print_r($config, 1));
    //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$this->McPay', print_r($this->McPay,1));

    $content = file_get_contents($filename);
    if (strpos($filename, 'check.js') !== FALSE) {
      $repl    = array(
        'payformid'             => $config['payformid'],
        'mcpay_card_token-form' => $config['payformid'],
      );
      $content = strtr($content, $repl);
    }

    echo $content;
  }

  /**
   * getFontAction
   *
   * @return bool
   */
  public function getImageAction()
  {
    $this->container->get('front')->Plugins()->ViewRenderer()->setNoRender();

    $this->Response()->clearAllHeaders();
    $this->Response()->setHeader('Content-Type', 'image/png');
    //header('Content-Type: image/png');

    $req = $this->Request()->getParams();
    if (empty($req['sn'])) {
      if ($this->debug) $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->Req empty: '.print_r($req['sn'], 1));
      $this->Response()->setHttpResponseCode(403);
      die('403');
    }

    $filename = MCPAY_BASE_PATH.'img/'.$req['sn'].'.png';
    if (!file_exists($filename)) {
      if ($this->debug) $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->File not found: '.print_r($filename, 1));
      $this->Response()->setHttpResponseCode(404);
      die('404');
    }
    $content = file_get_contents($filename);
    echo $content;
  }

  /**
   * getCfg
   *
   * @return mixed
   */
  public function getCfg()
  {
    $plugin = $this->container->get('kernel')->getPlugins()['MicropaymentApi'];
    $plugin->init();
    $config = $plugin->getCfg();
    $this->McPay->setConfig($config); // set in this instance too
    return $config;
  }

  /**
   * getInfos
   *
   * @param $pm
   *
   * @return array
   */
  public function getInfos($pm)
  {
    $amount = Shopware()->Session()->sBasketAmount;
    $currency = 'EUR'; // default
    $countryISO = 'DE'; // default

    $curId = Shopware()->Session()->sBasketCurrency;
    if (!empty($curId)) {
      $sql = 'SELECT * FROM `s_core_currencies` WHERE `id` = "'.$curId.'" ';
      $dat = Shopware()->Db()->fetchAll($sql);
      $currency = $dat[0]['currency'];
    }

    $countryId = Shopware()->Session()->sCountry;
    if (!empty($curId)) {
      $sql = 'SELECT * FROM `s_core_countries` WHERE `id` = "'.$countryId.'" ';
      $dat = Shopware()->Db()->fetchAll($sql);
      $countryISO = $dat[0]['countryiso'];
    }

    $xmlObj     = simplexml_load_file(__DIR__.'/../../plugin.xml');
    $chksumData = array(
      'amount'        => $amount * 100,
      'currency'      => $currency,
      'iso'           => $countryISO,
      'shoptype'      => 'shopware',
      'shopversion'   => Shopware()->Config()->version,
      'plugintype'    => 'API',
      'pluginversion' => (string)$xmlObj->version,
      'pm'            => $pm,
      'refid'         => session_id(),
    );
    return $chksumData;
  }

  /**
   * saveFields
   *
   * @param $fields
   * @param $paymentID
   *
   * @return false|\Zend_Db_Statement_Pdo
   * @throws \Zend_Db_Adapter_Exception
   */
  public static function saveFields($fields, $paymentID)
  {
    $userID = Shopware()->Session()->offsetGet('sUserId');
    $sql  = 'SELECT * FROM `s_plugin_mcpay_paydata` WHERE `userID` = "'.$userID.'" AND `pm` = "'.$paymentID.'" ';
    $dat = Shopware()->Db()->fetchAll($sql);

    $data = serialize($fields);
    if (!empty($dat)){
      $sql = 'UPDATE `s_plugin_mcpay_paydata` SET `data` = ? WHERE `userID` = "'.$userID.'" AND `pm` = "'.$paymentID.'" ';
      $params = array($data);
    } else {
      $sql = 'INSERT INTO `s_plugin_mcpay_paydata` (`userID`, `pm`, `data`) VALUES (?,?,?)';
      $params = array($userID, $paymentID, $data);
    }
    return Shopware()->Db()->query($sql, $params);
  }

  /**
   * loadFields
   *
   * @param $paymentID
   *
   * @return mixed
   */
  public static function loadFields($paymentID)
  {
    $userID = Shopware()->Session()->offsetGet('sUserId');
    $sql = 'SELECT * FROM `s_plugin_mcpay_paydata` WHERE `userID` = "'.$userID.'" AND `pm` = "'.$paymentID.'" ';
    $dat = current(Shopware()->Db()->fetchAll($sql));
    $data = unserialize($dat['data']);
    return $data;
  }

  /**
   * deleteFields
   *
   * @param $paymentID
   *
   * @return false|\Zend_Db_Statement_Pdo
   * @throws \Zend_Db_Adapter_Exception
   */
  public static function deleteFields($paymentID)
  {
    $userID = Shopware()->Session()->offsetGet('sUserId');
    $sql  = 'DELETE FROM `s_plugin_mcpay_paydata` WHERE `userID` = "'.$userID.'" AND `pm` = "'.$paymentID.'" ';
    return Shopware()->Db()->query($sql);
  }

}
