<?php
/**
 * Micropayment GmbH
 *
 * @link       http://www.micropayment.de
 * @copyright  Copyright (c) 2020, Micropayment GmbH
 * @author     Tobias Eilers
 * @package    Shopware
 * @subpackage Plugins
 */

use Shopware\Components\CSRFWhitelistAware;

class Shopware_Controllers_Frontend_MicropaymentOFGateway extends Shopware_Controllers_Frontend_Payment implements CSRFWhitelistAware
{
  var $McPay;
  var $cfg;

  /**
   * function to deactivate CSRF token
   * validation for specified actions
   *
   * @return array of ignored actions
   */
  public function getWhitelistedCSRFActions()
  {
    return array(
      'response',
      'success',
      'gateway',
      'notify',
      'getScript',
      'getStyle',
      'getstyle',
      'getscript'
    );
  }

  /**
   * Shopware_Controllers_Frontend_MicropaymentOF constructor.
   *
   * @param \Enlight_Controller_Request_Request   $request
   * @param \Enlight_Controller_Response_Response $response
   *
   * @throws \Enlight_Event_Exception
   * @throws \Enlight_Exception
   */
  function __construct(Enlight_Controller_Request_Request $request = NULL, Enlight_Controller_Response_Response $response = NULL)
  {
    define('MCPAY_BASE_PATH', dirname(__FILE__).'/../../mcpay/');
    require_once(MCPAY_BASE_PATH.'classes/class.mcpay.php');
    $this->McPay = new mcpay();
    $this->cfg   = $this->getCfg();
//    $config      = $this->Config();
//    $this->cfg   = $this->getCfg($config);
    
    //echo '<pre>'.print_r($this->McPay,true).'</pre>';

    parent::__construct($request, $response);
  }

  /**
   * getCfg
   *
   * @return mixed
   */
  public function getCfg()
  {
    $plugin = Shopware()->Container()->get('kernel')->getPlugins()['MicropaymentOFApi'];
    $plugin->init();
    $config                 = $plugin->getCfg();
    $this->McPay->setConfig($config); // set in this instance too
    return $config;
  }

  /**
   * recurringAction
   *
   * @throws \Exception
   */
  public function recurringAction()
  {
    $this->container->get('front')->Plugins()->ViewRenderer()->setNoRender();
    $params = $this->Request()->getParams();
    if ($this->McPay->debug) $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.': '.print_r($params, 1));

    if (!empty($params['orderId'])) {
      $orderId = $params['orderId'];

      $sql     = 'SELECT s_core_paymentmeans.id, s_order.userID 
              FROM s_order, s_core_paymentmeans 
              WHERE s_order.paymentID = s_core_paymentmeans.id 
              AND s_order.id = ?';
      $payData = current(Shopware()->Db()->fetchAll($sql, array($orderId)));

      $paymentID                          = $payData['id'];
      $userID                             = $payData['userID'];
      $sql                                = 'SELECT * FROM `s_plugin_mcpay_paydata` WHERE `userID` = "'.$userID.'" AND `pm` = "'.$paymentID.'" ';
      $dat                                = current(Shopware()->Db()->fetchAll($sql));
      $data                               = unserialize($dat['data']);
      $data['reuse']                      = 1;
      Shopware()->Session()->mcpay_fields = $data;
      $this->cfg['start3Dreuse']          = false;
      if ($this->McPay->debug) $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->Order('.$orderId.')->User('.$userID.'): '.print_r($data, 1).print_r($this->cfg,1));

      return $this->indexAction(FALSE);
    }
  }

  /**
   * indexAction
   *
   * @param bool $doRedirect
   *
   * @throws \Exception
   */
  public function indexAction($doRedirect = true)
  {
    //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$this->request', print_r($this->request->getParams(),1));

    $cfg                = $this->cfg;
    $user               = $this->getUser();
    $userData           = $this->getUserData($user, $cfg);
    $cfg['extraParams'] = array(
      'sessID' => Shopware()->Session()->sessionId,
    );
    //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$user', print_r($user,1));
    $this->McPay->debug = true;

    // PaymentMethode
    $activePayment                     = preg_replace('/mcpayofapi_/', '', $this->getPaymentShortName());
    $userData['chksum']['pm']          = $activePayment;
    Shopware()->Session()->McPayOFAPIChkSum = serialize($userData['chksum']);
    if ($this->McPay->debug) $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->config('.$activePayment.'): '.print_r($cfg, 1).print_r($userData, 1));

    $forwardURL = '';
    switch ($activePayment) {

      case 'ccard':
        $res = $this->doCCARD($userData, $doRedirect);
        break;
      case 'sepa':
        $res = $this->doSEPA($userData, $doRedirect);
        break;
      case 'prepay':
        $res = $this->doPREPAY($userData, $doRedirect);
        break;
      case 'invoice':
        $res = $this->doPREPAY($userData, $doRedirect);
        break;
      default:
        break;
    }

    if ($doRedirect && !empty($forwardURL)) {
      header('Location: '.$forwardURL);
    }

    return;
  }

  /**
   * doPREPAY
   *
   * @param      $userData
   * @param bool $doRedirect
   *
   * @return \stdClass|string|void
   * @throws \Exception
   */
  private function doPREPAY($userData, $doRedirect = true)
  {
//    echo '<pre>'.print_r($this,true).'</pre>';
//    echo '<pre>'.print_r($userData,true).'</pre>';
//    exit();

    try {
      $res = $this->McPay->bookPrepay($userData, $this->cfg);

      if ($res->resCreate->error == 0) { // create payment
        if ($res->resCreate->status == 'INIT') { // init payment

          $prepayText = $res->prepayTextDE;
          if ($res->custObj->lang != 'de') {
            $prepayText = $res->prepayTextEN;
          }
          Shopware()->Session()->McPayAPIPrepayText = $prepayText;

          // call notify internal
          $post = array(
            'function' => 'init',
            'orderid'  => $userData['order_id'],
            'auth'     => $res->resCreate->payToken,
            'amount'   => $userData['amount'],
            'currency' => $userData['currency'],
            'testmode' => $this->cfg['testMode'],
            'comment'  => $prepayText,
            'intresp'  => 1,
          );
          $res  = $this->notifyAction($post);

          // jump to thank you page
          $this->redirect(
            array(
              'controller'  => 'MicropaymentOFGateway',
              'action'      => 'success',
              'forceSecure' => 1,
              //'sTarget'     => 'checkout',
            )
          );
        } else {
          // jump to error page
        }
      } else {
        // jump to error page
      }

    } catch (Exception $e) {
      if ($this->McPay->debug) $this->McPay->log->log(__CLASS__.'->'.__FUNCTION__, print_r($res, 1)); // log error
      if ($this->McPay->debug) $this->McPay->log->log(__CLASS__.'->'.__FUNCTION__, $e->getMessage().' - '.$e->getCode()); // log error
      //die($e->getMessage());
      $errormsg                             = $e->getMessage();
      Shopware()->Session()->LastMcPayError = $errormsg;
      $this->redirect(
        array(
          'controller'  => 'MicropaymentOFGateway',
          'action'      => 'error',
          'forceSecure' => 1,
          //'sTarget'     => 'checkout',
        )
      );
    }
    return $res;
  }

  /**
   * doSEPA
   *
   * @param      $userData
   * @param bool $doRedirect
   *
   * @return array|bool|\stdClass|string|void
   * @throws \Exception
   */
  private function doSEPA($userData, $doRedirect = true)
  {
    if (!empty(Shopware()->Session()->mcpay_fields)) {
      //echo '<pre>'.print_r(Shopware()->Session()->mcpay_fields,1).'</pre>';
      // try to book
      try {
        $fields  = Shopware()->Session()->mcpay_fields;
        $payData = array(
          'holder' => $fields['mcpay_sepa_holder'],
          'iban'   => $fields['mcpay_sepa_iban'],
          'bic'    => $fields['mcpay_sepa_bic'],
          'reuse'  => $fields['reuse'],
        );

        $res = $this->McPay->bookSepa($userData, $this->cfg, $payData);

        if ($res->resCreate->error == 0) { // create payment
          if ($res->resApprove->error == 0 && $res->resApprove->status == 'APPROVED') { // approve payment

            $mandatText = $res->mandateTextDE;
            if ($res->custObj->lang != 'de') {
              $mandatText = $res->mandateTextEN;
            }
            Shopware()->Session()->McPayAPIMandatText = $mandatText;

            // call notify internal
            $post = array(
              'function' => 'billing',
              'orderid'  => $userData['order_id'],
              'auth'     => $res->resCreate->payToken,
              'amount'   => $userData['amount'],
              'currency' => $userData['currency'],
              'testmode' => $this->cfg['testMode'],
              'comment'  => $mandatText,
              'intresp'  => 1,
            );
            $res  = $this->notifyAction($post);

            // jump to thank you page
            if (!$doRedirect) return; // for recurring call
            $this->redirect(
              array(
                'controller'  => 'MicropaymentOFGateway',
                'action'      => 'success',
                'forceSecure' => 1,
                //'sTarget'     => 'checkout',
              )
            );
          } else {
            // jump to error page
          }
        } else {
          // jump to error page
        }

      } catch (Exception $e) {
        if ($this->McPay->debug) $this->McPay->log->log(__CLASS__.'->'.__FUNCTION__, print_r($res, 1)); // log error
        if ($this->McPay->debug) $this->McPay->log->log(__CLASS__.'->'.__FUNCTION__, $e->getMessage().' - '.$e->getCode()); // log error
        //die($e->getMessage());
        $errormsg                             = $e->getMessage();
        Shopware()->Session()->LastMcPayError = $errormsg;
        $this->redirect(
          array(
            'controller'  => 'MicropaymentOFGateway',
            'action'      => 'error2',
            'forceSecure' => 1,
            //'sTarget'     => 'checkout',
          )
        );
      }

    } else {
      $this->redirect(
        array(
          'controller'  => 'checkout',
          'action'      => 'shippingPayment',
          'forceSecure' => 1,
          'sTarget'     => 'checkout',
        )
      );
    }
    return $res;
  }

  /**
   * doCCARD
   *
   * @param      $userData
   * @param bool $doRedirect
   *
   * @return bool|\stdClass|string|void
   * @throws \Exception
   */
  private function doCCARD($userData, $doRedirect = true)
  {
    if (!empty(Shopware()->Session()->mcpay_fields)) {
      // try to book
      try {
        $fields = Shopware()->Session()->mcpay_fields;
        //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$fields', print_r($fields,1));
        $payData = array(
          'holder'   => $fields['mcpay_card_holder'],
          'month'    => $fields['mcpay_card_month'],
          'year'     => $fields['mcpay_card_year'],
          'pan'      => $fields['mcpay_card_pan'],
          'cvc'      => $fields['mcpay_card_cvc'],
          'token'    => $fields['mcpay_card_token'],
          'remember' => $fields['mcpay_card_remember'],
          'reuse'    => $fields['reuse'],
          '3Derror'  => Shopware()->Session()->mcpay3DError,
        );

        $config                  = $this->cfg;
        $config['returnURL']     = $this->Front()->Router()->assemble(array(
          'forceSecure' => 1,
          'action'      => 'threedsecure',
          'orderID'     => $userData['order_id'],
        ));
        $config['returnURLNO3D'] = $this->Front()->Router()->assemble(array(
          'forceSecure' => 1,
          'action'      => 'nothreedsecure',
          'orderID'     => $userData['order_id'],
          'sessID'      => '',
        ));
        $config['card_token']    = $fields['mcpay_card_token'];

        $res = $this->McPay->bookCCard($userData, $config, $payData);
        //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$res', print_r($fields, 1).print_r($payData, 1).print_r($config, 1).print_r($res, 1));

        if (!empty(Shopware()->Session()->mcpay_fields['pmid'])) {
          $paymentID = Shopware()->Session()->mcpay_fields['pmid'];
          $this->updateFields($paymentID);
        }

        if ($res->trxInfo->secureStatus != 'START3D') {
          // call notify internal
          $post = array(
            'function' => 'billing',
            'orderid'  => $userData['order_id'],
            'auth'     => $res->trxInfo->transactionId,
            'amount'   => $userData['amount'],
            'currency' => $userData['currency'],
            'testmode' => $this->cfg['testMode'],
            'intresp'  => 1,
          );
          $res  = $this->notifyAction($post);

          Shopware()->Session()->mcpay_fields = array(); // remove entered paydata

          // jump to thank you page
          if (!$doRedirect) return; // for recurring call
          $this->redirect(
            array(
              'controller'  => 'MicropaymentOFGateway',
              'action'      => 'success',
              'forceSecure' => 1,
              //'sTarget'     => 'checkout',
            )
          );
        } else if (!empty($res->forward)) {
          Shopware()->Session()->McPayUserData = $userData;
          $this->redirect($res->forwardURL);
        }

      } catch (Exception $e) {
        if ($this->McPay->debug) $this->McPay->log->log(__CLASS__.'->'.__FUNCTION__, print_r($res, 1)); // log error
        if ($this->McPay->debug) $this->McPay->log->log(__CLASS__.'->'.__FUNCTION__, $e->getMessage().' - '.$e->getCode()); // log error
        //die($e->getMessage());
        $errormsg                             = $e->getMessage();
        Shopware()->Session()->LastMcPayError = $errormsg;
        Shopware()->Session()->mcpay_fields   = array(); // remove entered paydata
        $this->redirect(
          array(
            'controller'  => 'MicropaymentOFGateway',
            'action'      => 'error3',
            'forceSecure' => 1,
            //'sTarget'     => 'checkout',
          )
        );
      }

    } else {
      if ($this->McPay->debug) $this->McPay->log->log(__CLASS__.'->'.__FUNCTION__, 'Redirect to Checkout Shipping due to missing mcpay_fields.'); // log error
      $this->redirect(
        array(
          'controller'  => 'checkout',
          'action'      => 'shippingPayment',
          'forceSecure' => 1,
          'sTarget'     => 'checkout',
        )
      );
    }
    return $res;
  }

  /**
   * threedsecureAction
   *
   * @throws \Exception
   */
  public function threedsecureAction()
  {
    if (!empty($_GET['orderID'])) {
      $order_id = $_GET['orderID'];
    } else {
      echo 'OrderID missing!';
      exit();
    }

    if (!empty($_REQUEST['check3dId'])) {
      $check3dId = $_REQUEST['check3dId'];
    } else if (!empty($_GET['check3dId'])) {
      $check3dId = $_GET['check3dId'];
    } else if (!empty($_POST['check3dId'])) {
      $check3dId = $_POST['check3dId'];
    } else {
      echo '3D Secure failed!';
      exit();
    }

    try {
      $res = $this->McPay->verify3D($check3dId, $this->cfg, Shopware()->Session()->McPayUserData);
      //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$res', print_r($res, 1));
      if ($this->McPay->debug) $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->verify3D: '.$check3dId.print_r($res, 1));

      if (isset($res->error) && $res->error == 0) {

        $sessionID = $this->McPay->unifyID($order_id);
        $info      = $this->McPay->sessionGet($sessionID);
        if ($this->McPay->debug) $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->info: '.print_r($info, 1));

        // call notify internal
        $post = array(
          'function' => 'billing',
          'orderid'  => $order_id,
          'auth'     => $res->transactionId,
          'amount'   => $info->amount,
          'currency' => $info->currency,
          'testmode' => $this->cfg['testMode'],
          'intresp'  => 1,
        );
        $res  = $this->notifyAction($post);
        if ($this->McPay->debug) $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->notifyAction: '.print_r($res, 1));

        Shopware()->Session()->mcpay_fields = array(); // remove entered paydata
        Shopware()->Session()->mcpay3DError = FALSE;

        // jump to thank you page
        $this->redirect(
          array(
            'controller'  => 'MicropaymentOFGateway',
            'action'      => 'success',
            'forceSecure' => 1,
            //'sTarget'     => 'checkout',
          )
        );
      } else {
        $errormsg = $res->transactionResultMessage;
        if ($this->McPay->debug) $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->verify3D->error: '.$errormsg);
        Shopware()->Session()->LastMcPayError = $errormsg;
        Shopware()->Session()->mcpay_fields   = array(); // remove entered paydata
        Shopware()->Session()->mcpay3DError   = TRUE;
        $this->redirect(
          array(
            'controller'  => 'MicropaymentOFGateway',
            'action'      => 'error4',
            'forceSecure' => 1,
            //'sTarget'     => 'checkout',
          )
        );
      }
    } catch (Exception $e) {
      $errormsg = $e->getMessage();
      if ($this->McPay->debug) $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->verify3D->error: '.$errormsg);
      Shopware()->Session()->LastMcPayError = $errormsg;
      Shopware()->Session()->mcpay_fields   = array(); // remove entered paydata
      $this->redirect(
        array(
          'controller'  => 'MicropaymentOFGateway',
          'action'      => 'error5',
          'forceSecure' => 1,
          //'sTarget'     => 'checkout',
        )
      );
    }
  }

  public function nothreedsecureAction()
  {
    echo __FUNCTION__;
  }

  public function transactionCreateAction()
  {
    $_POST['action'] = 'transactionCreate';
    return $this->responseAction();
  }


  public function updateTempID($tempID, $orderNumber)
  {
    $sql = '
            UPDATE s_order
            SET  temporaryID = "'.$tempID.'"
            WHERE ordernumber="'.$orderNumber.'" 
        ';
    Shopware()->Db()->executeQuery($sql);
  }

  public function getOrderIDByOrdernumber($orderNr)
  {
    $db = Shopware()->Db();
    $sql = 'SELECT `id` FROM `s_order` WHERE `ordernumber` = :orderNr ';
    return $db->fetchOne($sql, ['orderNr' => $orderNr]);
  }

  /**
   * responseAction
   *
   * @throws \Exception
   */
  public function responseAction()
  {
    $crlf = "\r\n";
    if ($this->McPay->debug) $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->post'.print_r($_REQUEST, 1));
    if (empty($_REQUEST['sessionId'])) {
      echo 'status=error'.$crlf.'message=sessionId cant be empty.';
      return;
    }

    $request = $this->Request();
    $params  = $request->getParams();
    if ($this->McPay->debug) $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->$params'.print_r($params, 1));
    // filter actions not needed
    $importantActions = array('transactionCreate', 'transactionRefund', 'transactionChargeback', 'transactionBackpay');
    if (!in_array($_POST['action'], $importantActions)) {
      echo 'status=ok'.$crlf.'message=Action '.$_POST['action'].'->'.$params['status'].' ignored.';
      return;
    }
    if (empty($params['type'])) {
      $params['type'] = $_POST['action'];
    }
    if (empty($params['testMode'])) {
      $params['testMode'] = 0;
    }
    $data = array(
      'function' => $params['type'],
      'orderid'  => $params['sessionId'],
      'testmode' => $params['testMode'],
      'auth'     => $params['transactionId'],
      'amount'   => $params['amount'],
      'intresp'  => 1,
    );
    if (in_array($params['type'], array('CREATE', 'CHANGE'))) {
      $data['amount'] = $params['orderAmount'];
    }
    if (in_array($params['type'], array('PAYIN', 'REFUND'))) {
      $data['amount'] = $params['paidAmount'];
    }
    $res = $this->notifyAction($data);
    echo $res;
  }

  /**
   * notifyAction
   *
   * @param array $post
   *
   * @throws \Exception
   */
  public function notifyAction($post = array())
  {
    $return = '';
    $crlf   = "\r\n";
    $cfg    = $this->cfg;
    if (empty($post)) {
      $post = $_GET;
    }

    if (!empty($post['secretfield'])) {
      $secretfield = $cfg['secretfield'];
      if ($secretfield != $post['secretfield']) {
        $return = 'status=error'.$crlf.'message=Communication is insecure.';
      }
    }

    // check mandatory params
    if (empty($post['function'])) $return = 'status=error'.$crlf.'message=Function missing.';
    else                            $func = $post['function'];

    if ($func != 'custinfo') {
      if ($this->McPay->debug) $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->post'.print_r($post, 1));

      if (empty($post['intresp'])) $return = 'status=error'.$crlf.'message=Please use responseAction as notification url.';

      if (empty($post['orderid'])) $return = 'status=error'.$crlf.'message=OrderID missing.';
      else                          $orderId = $post['orderid'];

      if (empty($post['auth'])) $return = 'status=error'.$crlf.'message=Auth missing.';
      else                          $authId = $post['auth'];

      if (empty($post['amount'])) $return = 'status=error'.$crlf.'message=Amount missing.';
      else                          $amount = $post['amount'];
      /*
      if (empty($post['currency'])) $return = 'status=error'.$crlf.'message=Currency missing.';
      else                          $currency = $post['currency'];
      */
      if (!isset($post['testmode'])) $return = 'status=error'.$crlf.'message=Testmode missing.';
      else                          $testmode = $post['testmode'];

      $currency = 'EUR';
      if (!empty($post['currency'])) $currency = $post['currency'];

      $theAmount = sprintf('%1.2f', $amount / 100).' '.$currency;

      if (!empty($orderId)) {
        // extract Order ID
        if (strpos($orderId, '-') !== FALSE) {
          $parts   = explode('-', $orderId);
          $orderId = $parts[0];
          $custId  = $parts[1];
        }
        // get transactionID for normal handling below
        $order = Shopware()->Modules()->Order();
        $oID = $this->getOrderIDByOrdernumber($orderId);
        $orderData = $order->getOrderById($oID);
        $orderNumber = $orderId;
        $transactionId = $orderData['transactionID'];
        $payAuth = $authId;
        $authId = 'AUTHID';
      } else {
        $return = 'status=error'.$crlf.'message=OrderID missing!';
      }
    } else {
      if (empty($post['ak'])) $return = 'status=error'.$crlf.'message=Missing KEY!';
      else                      $ak = $post['ak'];
    }

    if (!empty($return)) {
      echo $return;
      return $return;
    }
    // default error after pre-checks
    $return = 'status=error'.$crlf.'message=Unknown Error';

    switch ($func) {

      // status call
      case 'custinfo':
        //echo '<pre>'.print_r($cfg,1).'</pre>';
        $accessKey = $cfg['accessKey'];
        //echo $accessKey.' == '.$ak;
        if ($accessKey == $ak) {
          if (!empty($post['log'])) {
            //echo $this->McPay->readLogFile($this->McPay->log->error_log, TRUE);
            echo $this->McPay->readerLogFile($this->McPay->log->error_log);
            exit();
          } else {
            $plugin = $this->container->get('kernel')->getPlugins()['MicropaymentOFApi'];
            $plugin->init();
            $config = $plugin->getCfg();
            $xmlObj = simplexml_load_file(__DIR__.'/../../plugin.xml');
            $infos  = 'Shopversion: '.Shopware()->Config()->version.'<br>';
            $infos  .= 'Pluginversion: '.$xmlObj->version.'<br>';
            $infos  .= '<textarea>'.$xmlObj->asXML().'</textarea>';
            //echo '<pre>'.print_r($xmlObj,1).'</pre>';
            echo '<pre>'.print_r($infos, 1).'</pre>';
            echo '<pre>CFG:'.print_r($config, 1).'</pre>';
            $activePayment = $plugin->getActivePaymethods();
            echo '<pre>Paymentmeans:'.print_r($activePayment, 1).'</pre>';
            $return = '';
          }
        } else {
          $return = 'status=error'.$crlf.'message=Access denied!';
        }
        break;

      case 'BOOKING':
        try {
          //$transactionId = $orderId;
          //$comment = $this->getOrderComment($transactionId);
          $comment = $crlf.'Payment TRXID: '.$authId;
          $this->addOrderComment($transactionId, $comment);
        } catch (Exception $e) {
          $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->BOOKING: '.$e->getMessage());
        }
        $return = 'status=ok'.$crlf.'message=BOOKING done.'.$crlf;
        break;

      case 'CREATE':
      case 'CHANGE':
        try {
          //$transactionId = $orderId;
          $comment       = $crlf.'Prepayment '.$func.' TRXID: '.$authId.' Amount: '.$theAmount;
          $res           = $this->addOrderComment($transactionId, $comment);
        } catch (Exception $e) {
          $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->'.$func.': '.$e->getMessage());
        }
        $return = 'status=ok'.$crlf.'message='.$func.' done.'.$crlf;
        break;

      // billing call
      case 'billing':
      case 'init':
        //case 'BOOKING':
        //$transactionId   = $orderId;
        $paymentUniqueId = $authId;
        $paymentStatus   = 12; // default payment status is "12 Komplett bezahlt"
        $sendStatusMail = TRUE;
        if ($func == 'init') {
          $paymentStatus = 19; // delayed
          $sendStatusMail = FALSE;
        }

        // not working because its always one order to late. chicken-egg-problem
        // add comment to order comment to have it in mail
//        if (!empty($post['comment'])) {
//          Shopware()->Session()->sComment = $post['comment'];
//          $post['comment']                = ''; // clean comment to have it only in one field in backend
//        } else {
//          Shopware()->Session()->sComment = '';
//        }
        //$orderNumber = $this->saveOrder($transactionId, $paymentUniqueId, $paymentStatus);
        //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$orderNumber', print_r($orderNumber,1));
        $this->savePaymentStatus($transactionId, $paymentUniqueId, $paymentStatus, $sendStatusMail);

        $this->updateTempID($payAuth, $orderNumber);
        Shopware()->Session()->LastMcPayTrans = $paymentUniqueId;

        // Add Infos to Order
        $internalcomment = '';
        $params          = array(
          'o_attr1'         => $transactionId,
          'o_attr2'         => $payAuth,
          'o_attr3'         => $amount,
          'o_attr4'         => "Title: ".$this->Request()->getParam('title'),
          'o_attr5'         => "Testmode: ".$testmode,
          'comment'         => "AuthId: ".$payAuth,
          'internalcomment' => $internalcomment,
        );
        if (!empty($post['comment'])) {
          $params['comment'] .= $crlf.$post['comment'];
        }
        // save paytoken
        if (!empty($post['paytoken'])){
          $params['o_attr6'] = $post['paytoken'];
        }
        if ($paymentStatus == 12) { // if payment status "Komplett bezahlt"
          $params['cleareddate'] = date('Y-m-d H:i:s');
        }
        // add sepa trx id from BOOKING notification. This is special because BOOKING notify comes before BILLING and order doesnt exist at this time
//        $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->McPaySepaTRXID: '.Shopware()->Session()->McPayAPISepaTRXID);
//        if (!empty(Shopware()->Session()->McPayAPISepaTRXID)){
//          $params['comment'] .= $crlf.'Payment TRXID: '.Shopware()->Session()->McPayAPISepaTRXID;
//          Shopware()->Session()->McPayAPISepaTRXID = false;
//        }
        $this->addOrderInfos($transactionId, $params);

        $url    = $this->Front()->Router()->assemble(array(
          'forceSecure' => 1,
          'action'      => 'success',
        ));
        $return = 'status=ok'.$crlf // ok  | error
          .'url='.$url.$crlf
          .'target=_top'.$crlf
          .'forward=1'.$crlf
          .'message=OK'.$crlf;
        break;

      case 'payin': // payin prepayment
      case 'PAYIN':
        try {
          $this->cfg   = $this->getCfg();
          $this->McPay->switchPaymethod('prepay');
          $trx = $this->McPay->sessionGet($orderId);
          $post['paystatus'] = $trx->status;
          //echo '<pre>'.print_r($trx,true).'</pre>';

          //$transactionId   = $orderId;
          $res             = $this->getOrderAttributes($transactionId);
          $paymentUniqueId = $res['attribute2'];
          $paymentStatusId = '11'; // payment partially

          // paystatus = PAID oder OVERPAID
          $sendStatusMail = FALSE;
          if (in_array($post['paystatus'], array('PAID', 'OVERPAID'))) {
            //if ($amount >= $res['attribute3']) $paymentStatusId = '12'; // payment complete
            if ($trx->openAmount <= 0) {
              $paymentStatusId = '12'; // payment complete
              $sendStatusMail = TRUE;
            }
          }
          $this->savePaymentStatus($transactionId, $paymentUniqueId, $paymentStatusId, $sendStatusMail);
          // add comment
          $comment = $crlf.'Payin TRXID: '.$authId.' Amount: '.$theAmount;
          $this->addOrderComment($transactionId, $comment);
        } catch (Exception $e) {
          $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->payin: '.$e->getMessage());
        }
        $return = 'status=ok'.$crlf.'message=PAYIN done.'.$crlf;
        break;

      case 'expire': // expire prepayment
        try {
          //$transactionId   = $orderId;
          $res             = $this->getOrderAttributes($transactionId);
          $paymentUniqueId = $res['attribute2'];
          $paymentStatusId = '35'; // payment partially
          $sendStatusMail  = FALSE;
          $this->savePaymentStatus($transactionId, $paymentUniqueId, $paymentStatusId, $sendStatusMail);
          // add comment
          $comment = $crlf.'Expire TRXID: '.$authId.' Amount: '.$theAmount;
          $this->addOrderComment($transactionId, $comment);
        } catch (Exception $e) {
          $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->expire: '.$e->getMessage());
        }
        $return = 'status=ok'.$crlf.'message=EXPIRE done.'.$crlf;
        break;

      case 'storno': // chargeback sepa
      case 'transactionChargeback': // chargeback ccard
        try {
          //$transactionId   = $orderId;
          $res             = $this->getOrderAttributes($transactionId);
          $paymentUniqueId = $res['attribute2'];
          $paymentStatusId = '35'; // cancelled
          $sendStatusMail  = FALSE;
          $this->savePaymentStatus($transactionId, $paymentUniqueId, $paymentStatusId, $sendStatusMail);
          // add comment
          $comment = $crlf.'Storno TRXID: '.$authId.' Amount: '.$theAmount;
          $this->addOrderComment($transactionId, $comment);
        } catch (Exception $e) {
          $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->storno: '.$e->getMessage());
        }
        $return = 'status=ok'.$crlf.'message=STORNO done.'.$crlf;
        break;

      case 'backpay': // chargeback reversal
      case 'REFUNDREVERSAL'; // refund reversal
      case 'transactionBackpay': // backpay ccard
        try {
          //$transactionId   = $orderId;
          $res             = $this->getOrderAttributes($transactionId);
          $paymentUniqueId = $res['attribute2'];
          $paymentStatusId = '21'; // clarification
          $sendStatusMail  = FALSE;
          $this->savePaymentStatus($transactionId, $paymentUniqueId, $paymentStatusId, $sendStatusMail);
          // add comment
          $comment = $crlf.'Backpay TRXID: '.$authId.' Amount: '.$theAmount;
          $this->addOrderComment($transactionId, $comment);
        } catch (Exception $e) {
          $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->backpay: '.$e->getMessage());
        }
        $return = 'status=ok'.$crlf.'message=BACKPAY done.'.$crlf;
        break;

      case 'refund': // refund sepa
      case 'REFUND': // refund prepayment
      case 'transactionRefund': // refund ccard
        try {
          $this->cfg   = $this->getCfg();
          $this->McPay->switchPaymethod('prepay');
          $trx = $this->McPay->sessionGet($orderId);
          $post['paystatus'] = $trx->status;
          //echo '<pre>'.print_r($trx,true).'</pre>';

          // if paystatus == PAID or OVERPAID do nothing because prepayment
          if (in_array($post['paystatus'], array('PAID', 'OVERPAID'))) {
            break;
          }
          //$transactionId   = $orderId;
          $res             = $this->getOrderAttributes($transactionId);
          $paymentUniqueId = $res['attribute2'];
          $paymentStatusId = '20'; // recredit
          $sendStatusMail  = FALSE;
          $this->savePaymentStatus($transactionId, $paymentUniqueId, $paymentStatusId, $sendStatusMail);
          // add comment
          $comment = $crlf.'Refund TRXID: '.$authId.' Amount: '.$theAmount;
          $this->addOrderComment($transactionId, $comment);
        } catch (Exception $e) {
          $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->refund: '.$e->getMessage());
        }
        $return = 'status=ok'.$crlf.'message=Refund done.'.$crlf;
        break;

      case 'quit': // reversal SEPA only
        try {
          //$transactionId   = $orderId;
          $res             = $this->getOrderAttributes($transactionId);
          $paymentUniqueId = $res['attribute2'];
          $paymentStatusId = '35'; // process cancelled
          $sendStatusMail  = FALSE;
          $this->savePaymentStatus($transactionId, $paymentUniqueId, $paymentStatusId, $sendStatusMail);
        } catch (Exception $e) {
          $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->quit: '.$e->getMessage());
        }
        $return = 'status=ok'.$crlf.'message=QUIT done.'.$crlf;
        break;
    }

    $this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->return: '.$return);
    echo $return;
    return;
  }


  /**
   * successAction
   *
   * @throws \Exception
   */
  public function successAction()
  {
    // update customer IP address in DB
    $ip = $_SERVER['REMOTE_ADDR'];
    if (!empty($_SERVER['HTTP_X_FORWARDED_FOR'])){ // add this to config.php -> 'trustedProxies' => ['127.0.0.1'],
      $ip = $_SERVER['HTTP_X_FORWARDED_FOR'];
    }
    $ip = Shopware()->Container()->get('shopware.components.privacy.ip_anonymizer')->anonymize((string) $ip);
    $sql = 'UPDATE `s_order` SET `remote_addr` = ? WHERE `temporaryID` = ?';
    Shopware()->Db()->query($sql, array($ip, Shopware()->Session()->LastMcPayTrans));

    return $this->redirect(array(
        'controller'  => 'checkout',
        'action'      => 'finish',
        'forceSecure' => 1,
        'sUniqueID'   => Shopware()->Session()->LastMcPayTrans,
      )
    );
    unset(Shopware()->Session()->LastMcPayTrans);
  }

  /**
   * errorAction
   */
  public function errorAction()
  {
    Shopware()->Template()->addTemplateDir(dirname(__FILE__).'/../../Views/');
    $this->getCfg();
    $content                    = $this->McPay->getPoweredBy(FALSE, Shopware()->Session()->McPayOFAPIChkSum, -1, Shopware()->Session()->LastMcPayError);
    $content                    .= htmlentities(Shopware()->Session()->LastMcPayError);
    $this->View()->ErrorMessage = $content;
  }

  /**
   * updateFields
   *
   * @param $paymentID
   *
   * @return false|\Zend_Db_Statement_Pdo
   * @throws \Zend_Db_Adapter_Exception
   */
  public static function updateFields($paymentID)
  {
    $userID = Shopware()->Session()->offsetGet('sUserId');
    $sql    = 'SELECT * FROM `s_plugin_mcpay_paydata` WHERE `userID` = "'.$userID.'" AND `pm` = "'.$paymentID.'" ';
    $dat    = current(Shopware()->Db()->fetchAll($sql));
    $fields = unserialize($dat['data']);

    $fields['mcpay_card_reuse'] = 1;
    $data                       = serialize($fields);
    $sql                        = 'UPDATE `s_plugin_mcpay_paydata` SET `data` = ? WHERE `userID` = "'.$userID.'" AND `pm` = "'.$paymentID.'" ';
    $params                     = array($data);

    return Shopware()->Db()->query($sql, $params);
  }

  /**
   * addOrderInfos
   *
   * @param $transactionID
   * @param $params
   *
   * @return bool|void
   */
  function addOrderInfos($transactionID, $params)
  {
    try {
      $orderModel = Shopware()->Models()->getRepository('Shopware\Models\Order\Order')->findOneBy(array('transactionId' => $transactionID));
      if (empty($orderModel)) {
        return FALSE;
      }
      $oldValue = $orderModel->getInternalComment();
      if (!empty($params['internalcomment'])) {
        $params['internalcomment'] = date('d.m.Y H:i:s')."\n".$params['internalcomment']."\n \n".$oldValue;
      } else {
        $params['internalcomment'] = $oldValue;
      }
      // Mapping database -> model
      $orderMappings = array(
        'ordernumber'          => 'number',
        'userID'               => 'customerId',
        'invoice_amount'       => 'invoiceAmount',
        'invoice_amount_net'   => 'invoiceAmountNet',
        'invoice_shipping'     => 'invoiceShipping',
        'invoice_shipping_net' => 'invoiceShippingNet',
        'ordertime'            => 'orderTime',
        'status'               => 'status',
        'cleared'              => 'cleared',
        'paymentID'            => 'paymentId',
        'transactionID'        => 'transactionId',
        'comment'              => 'comment',
        'customercomment'      => 'customerComment',
        'internalcomment'      => 'internalComment',
        'net'                  => 'net',
        'taxfree'              => 'taxFree',
        'partnerID'            => 'partnerId',
        'temporaryID'          => 'temporaryId',
        'referer'              => 'referer',
        'cleareddate'          => 'clearedDate',
        'trackingcode'         => 'trackingCode',
        'language'             => 'languageIso',
        'dispatchID'           => 'dispatch',
        'currency'             => 'currency',
        'currencyFactor'       => 'currencyFactor',
        'subshopID'            => 'shopId',
        'remote_addr'          => 'remoteAddress',
      );

      $attributeMapping = array(
        'o_attr1' => 'attribute1',
        'o_attr2' => 'attribute2',
        'o_attr3' => 'attribute3',
        'o_attr4' => 'attribute4',
        'o_attr5' => 'attribute5',
        'o_attr6' => 'attribute6');

      $newData           = array();
      $attribute         = array();
      $params['o_attr6'] = "MCPAYAPI";
      //order mapping
      foreach ($orderMappings AS $k => $v) {
        if (isset($params[$k])) $newData[$v] = $params[$k];
      }
      //attribute mapping
      foreach ($attributeMapping as $k => $v) {
        if (isset($params[$k])) $attribute[$v] = $params[$k];
      }
      if (!empty($attribute)) {
        $newData['attribute'] = $attribute;
      }

      if (isset($params['cleared'])) {
        $sql = 'UPDATE `s_order` SET `cleared` = ? WHERE `transactionID` = ?';
        Shopware()->Db()->query($sql, array((int)$params['cleared'], $transactionID));
      }

      // populate Model with data
      $orderModel->fromArray($newData);
      Shopware()->Models()->persist($orderModel);
      // save to database
      Shopware()->Models()->flush();
    } catch (Exception $e) {
      $plugin = $this->container->get('kernel')->getPlugins()['MicropaymentOFApi'];
      $plugin->Logging('getActivePaymethods | '.$e->getMessage());
      return;
    }
  }

  /**
   * getOrderAttributes
   *
   * @param $transactionID
   *
   * @return bool|object|\Shopware\Models\Order\Order|void|null
   */
  function getOrderAttributes($transactionID)
  {
    try {
      $sql = 'SELECT * FROM `s_order_attributes` WHERE `attribute1` = ?';
      $dat = Shopware()->Db()->fetchAssoc($sql, $transactionID);
      return current($dat);
    } catch (Exception $e) {
      $mpBootstrap = Shopware()->Plugins()->Frontend()->MicropaymentOF();
      $mpBootstrap->Logging('getActivePaymethods | '.$e->getMessage());
      return;
    }
  }

  /**
   * formatNumber
   *
   * @param $value
   *
   * @return string
   */
  public function formatNumber($value)
  {
    return sprintf('%1.2f', $value);
  }

  /**
   * getUserData
   *
   * @param $user
   * @param $config
   *
   * @return array
   */
  function getUserData($user, $config)
  {
    //echo '<pre>'.print_r($user,1).'</pre>';
    //echo '<pre>'.print_r($config,1).'</pre>';

    $countryISO = Shopware()->Db()->fetchOne("
          SELECT `countryiso` FROM `s_core_countries`
          WHERE `id` = ?
          ", array($user['billingaddress']['countryID']));

    $custID                               = $user['additional']['user']['userID'];
    $orderId                              = $this->createPaymentUniqueId();
    //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$orderId', print_r($orderId,1));
    if (empty($custID)) { // guest order
      $custID = 'guest-'.$orderId;
    } else {
      //$orderId .= '-'.$custID;
    }
    $custID = $this->McPay->unifyID($custID);

    // use real ordernumber
    $transactionId = $this->createPaymentUniqueId();
    $paymentUniqueId = 'AUTHID';
    $paymentStatus = 17; // open
    //$paymentStatus = 19; // delayed
    $orderId = $this->saveOrder($transactionId, $paymentUniqueId, $paymentStatus);

    $amount     = $this->formatNumber($this->getAmount());
    //$currency   = Shopware()->Currency()->getShortName();
    $currency   = $this->getCurrencyShortName();
    $xmlObj     = simplexml_load_file(__DIR__.'/../../plugin.xml');
    $chksumData = array(
      'amount'        => $amount * 100,
      'currency'      => $currency,
      'iso'           => $countryISO,
      'host'          => $_SERVER['HTTP_HOST'],
      'project'       => $config['project'],
      'shoptype'      => 'shopware',
      'shopversion'   => Shopware()->Config()->version,
      'plugintype'    => 'API',
      'pluginversion' => (string)$xmlObj->version,
      'refid'         => session_id(),
    );

    $shopContext = $this->get('shopware_storefront.context_service')->getShopContext();
    $lang = substr($shopContext->getShop()->getLocale()->getLocale(),0,2);

    $userData = array(
      'company'    => $user['billingaddress']['company'],
      'firstName'  => $user['billingaddress']['firstname'],
      'lastName'   => $user['billingaddress']['lastname'],
      'salutation' => 'MR',
      'street'     => $user['billingaddress']['street']." ".$user['shippingaddress']['streetnumber'],
      'zip'        => $user['billingaddress']['zipcode'],
      'city'       => $user['billingaddress']['city'],
      'country'    => $countryISO,
      'email'      => $user['additional']['user']['email'],
      'ip'         => $_SERVER['REMOTE_ADDR'],
      'id'         => $custID,
      'order_id'   => $orderId,
      'lang'       => $lang,
      'amount'     => ($amount * 100),
      'currency'   => $currency,
      'chksum'     => $chksumData, // request security
    );
    if (empty($userData['ip'])) $userData['ip'] = $_SERVER['REMOTE_ADDR']; // Falls IP Leer, dann aus dem Server holen
    if (!empty($_SERVER['HTTP_X_FORWARDED_FOR'])){ // add this to config.php -> 'trustedProxies' => ['127.0.0.1'],
      $userData['ip'] = $_SERVER['HTTP_X_FORWARDED_FOR'];
    }

    return $userData;
  }

  /**
   * addOrderComment
   *
   * @param $transactionID
   * @param $comment
   *
   * @return \Zend_Db_Statement_Pdo
   */
  public function addOrderComment($transactionID, $comment)
  {
    $sql = 'UPDATE `s_order` SET `comment` = CONCAT(`comment`, "?") WHERE transactionID="?"';
    return Shopware()->Db()->executeQuery($sql, array($comment, $transactionID));
  }

  /**
   * getOrderComment
   *
   * @param $transactionID
   *
   * @return array|false
   */
  public function getOrderComment($transactionID)
  {
    $sql = 'SELECT `comment` FROM `s_order` WHERE transactionID="'.$transactionID.'"';
    return Shopware()->Db()->fetchAll($sql);
  }

  /**
   * checkUpdateAction
   * check for plugin updates at micropayment
   */
  public function checkUpdateAction()
  {
    Shopware()->Template()->addTemplateDir(dirname(__FILE__).'/../../Views/');

    $plugin = $this->container->get('kernel')->getPlugins()['MicropaymentOFApi'];
    $plugin->init();
    $config  = $plugin->getCfg();
    $xmlObj  = simplexml_load_file(__DIR__.'/../../plugin.xml');
    $infos   = array(
      'project'       => $config['project'],
      'host'          => $_SERVER['HTTP_HOST'],
      'shoptype'      => 'shopware',
      'shopversion'   => Shopware()->Config()->version,//Shopware::VERSION,
      'plugintype'    => 'APIOF',
      'pluginversion' => (string)$xmlObj->version,
      'status'        => 'stable', // check for stable plugins only (all, stable, beta and alpha)
    );
    //$this->McPay->log->debug(__CLASS__.'->'.__FUNCTION__.'->infos: '.print_r($infos,1));
    $content = '<script src=\''.mcpay::getVersionURL().'?d='.serialize($infos).'\'></script>';
    $this->view->assign('content', $content);
  }

}
