<?php

namespace MicropaymentOFApi;

use Shopware\Components\Plugin;
use Shopware\Components\Plugin\Context\ActivateContext;
use Shopware\Components\Plugin\Context\DeactivateContext;
use Shopware\Components\Plugin\Context\InstallContext;

class MicropaymentOFApi extends Plugin
{
  var $McPay;

  public function init()
  {
    define('MCPAY_BASE_PATH', dirname(__FILE__).'/../../mcpay/');
    require_once(MCPAY_BASE_PATH.'classes/class.mcpay.php');
    $this->McPay = new \mcpay();
  }

  /**
   * install
   *
   * @param \MicropaymentOFApi\InstallContext $context
   */
  public function install(InstallContext $context)
  {
    try {
      $this->createTables();

      /** @var \Shopware\Components\Plugin\PaymentInstaller $installer */
      $installer = $this->container->get('shopware.plugin_payment_installer');

      define('MCPAY_BASE_PATH', dirname(__FILE__).'/mcpay/');
      require_once(MCPAY_BASE_PATH.'classes/class.mcpay.php');
      $this->McPay = new \mcpay();
      //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$this->McPay', print_r($this->McPay,1));

      $pm  = $this->getAvailablePaymethods();
      $pos = 1;
      foreach ($pm AS $k => $v) {
        $msg     = 'Installing '.$v['desc_en'].' ';
        $options = [
          'name'                  => "mcpayofapi_".$v['name'],
          'description'           => 'Micropayment Order First API '.$v['desc_de'],
          'action'                => 'MicropaymentOFGateway',
          'active'                => 0,
          'position'              => $pos,
          'additionalDescription' => $this->McPay->getPoweredBy().$this->McPay->getLogo($v['name'], $v['desc_de']),
        ];
        $installer->createOrUpdate($context->getPlugin(), $options);
        $pos++;
      }
    } catch (Exception $e) {
      $msg .= 'failed!';
      $this->logError($msg, $e);
    }
  }

  /**
   * @param UninstallContext $context
   */
  public function uninstall(Plugin\Context\UninstallContext $context)
  {
    $this->setActiveFlag($context->getPlugin()->getPayments(), FALSE);
  }

  /**
   * @param DeactivateContext $context
   */
  public function deactivate(Plugin\Context\DeactivateContext $context)
  {
    $this->setActiveFlag($context->getPlugin()->getPayments(), FALSE);
    $context->scheduleClearCache(DeactivateContext::CACHE_LIST_ALL);
  }

  /**
   * @param ActivateContext $context
   */
  public function activate(Plugin\Context\ActivateContext $context)
  {
    $this->setActiveFlag($context->getPlugin()->getPayments(), TRUE);
    $context->scheduleClearCache(ActivateContext::CACHE_LIST_ALL);
  }

  /**
   * @param Payment[] $payments
   * @param           $active bool
   */
  private function setActiveFlag($payments, $active)
  {
    $em = $this->container->get('models');

    foreach ($payments as $payment) {
      $payment->setActive($active);
    }
    $em->flush();
  }

  /**
   * getAvailablePaymethods
   *
   * @return array
   */
  public function getAvailablePaymethods()
  {
    return array(
      '0' => array(
        'name'    => 'prepay',
        'desc_de' => 'Vorkasse',
        'desc_en' => 'Prepayment',
      ),
//      '1' => array(
//        'name'    => 'invoice',
//        'desc_de' => 'Rechnung',
//        'desc_en' => 'Invoice',
//      ),
    );
  }

  /**
   * getActivePaymethods
   *
   * @return array|void
   */
  public function getActivePaymethods()
  {
    try {
      $dbErrors = '';

      $sql = 'SELECT * FROM `s_core_paymentmeans` WHERE `name` LIKE "mcpayofapi\_%";';
      try {
        $getOldPayments = Shopware()->Db()->fetchAssoc($sql);
        $dbErrors       .= Shopware()->Db()->getErrorMessage();

        return $getOldPayments;
      } catch (Exception $e) {
        if (empty($dbErrors)) {
          $this->Logging('createPayments Fetching old paymethods| '.$e->getMessage());
          return;
        } else {
          $this->Logging('createPayments Fetching old paymethods | '.$e->getMessage().' | DB-Errors: '.$dbErrors);
          return;
        }
      }
    } catch (Exception $e) {
      $this->Logging('getActivePaymethods | '.$e->getMessage());
      return;
    }
  }

  /**
   * getConfig
   *
   * @return array
   */
  public function getConfig()
  {
    $fields = $this->configFields();
    $conf   = array();
    foreach ($fields AS $k) {
      $conf[$k] = Shopware()->Config()->getByNamespace('micropaymentofapi', $k);
    }
    // set style and script url
    $conf['MCPAY_JSURL']  = Shopware()->Front()->Router()->assemble(array('forceSecure' => 1, 'action' => 'getScript', 'controller' => 'MicropaymentCcard')).'/sn/';
    $conf['MCPAY_CSSURL'] = Shopware()->Front()->Router()->assemble(array('forceSecure' => 1, 'action' => 'getStyle', 'controller' => 'MicropaymentCcard')).'/sn/';
    return $conf;
  }

  /**
   * configFields
   *
   * @return array
   */
  public function configFields()
  {
    return array(
      'MCPAY_PROJECTID',
      'MCPAY_ACCESSKEY',
      'MCPAY_TESTMODE',
      'MCPAY_DOMAIN',
      'MCPAY_THEME',
      'MCPAY_GFX',
      'MCPAY_BGCOLOR',
      'MCPAY_BGGFX',
      'MCPAY_SECRETFIELD',
      'MCPAY_DEBUG',
      'MCPAY_DEBUG_LOG',
      'MCPAY_START3D',
      'MCPAY_START3D_REUSE',
      'MCPAY_EXPIREDAYS',
      'MCPAY_PAYTEXT',
    );
  }

  /**
   * getCfg
   *
   * @return array
   */
  function getCfg()
  {
    $config    = $this->getConfig();
    $configMap = array(
      'MCPAY_ACCESSKEY'     => 'accessKey',
      'MCPAY_BGCOLOR'       => 'bgcolor',
      'MCPAY_BGGFX'         => 'bggfx',
      'MCPAY_CCARDFORMID'   => '',
      'MCPAY_DEBUG'         => 'debug',
      'MCPAY_DEBUG_LOG'     => '',
      'MCPAY_DOMAIN'        => 'suffix',
      'MCPAY_GFX'           => 'gfx',
      'MCPAY_PAYFORMID'     => '',
      'MCPAY_PROJECTID'     => 'project',
      'MCPAY_SECRETFIELD'   => 'secretfield',
      'MCPAY_SEPAFORMID'    => '',
      'MCPAY_TESTMODE'      => 'testMode',
      'MCPAY_THEME'         => 'theme',
      'MCPAY_JSURL'         => 'plugin_urljs',
      'MCPAY_CSSURL'        => 'plugin_urlcss',
      'MCPAY_START3D'       => 'start3D',
      'MCPAY_START3D_REUSE' => 'start3Dreuse',
      'MCPAY_EXPIREDAYS'    => 'expiredays',
      'MCPAY_PAYTEXT'       => 'paytext',
    );
    $cfg       = array();
    foreach ($configMap AS $k => $v) {
      if (empty($v)) continue;
      $cfg[$v] = $config[$k];
    }
    //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$cfg', print_r($cfg,1).print_r($config,1));
    $this->McPay->setConfig($cfg);

    return $cfg;
  }

  /**
   * createTables
   *
   * @throws \Zend_Db_Adapter_Exception
   */
  protected function createTables()
  {
    $sql = 'CREATE TABLE IF NOT EXISTS `s_plugin_mcpay_paydata`(
				`id` int(11) unsigned NOT NULL AUTO_INCREMENT,
				`userID` int(11) NOT NULL,
				`pm` varchar(10) NOT NULL,
				`data` mediumtext NOT NULL,
				PRIMARY KEY (`id`)
				) ENGINE=InnoDB DEFAULT CHARSET=utf8 AUTO_INCREMENT=1;
			';
    Shopware()->Db()->query($sql);
  }

  /**
   * logError
   *
   * @param $msg
   * @param $e
   */
  public static function logError($msg, $e)
  {
    self::Logging($msg.$e->getMessage());
  }

  /**
   * Logging
   *
   * @param $msg
   *
   * @return false|void|\Zend_Db_Statement_Pdo
   */
  public static function Logging($msg)
  {
    try {
      $sql = 'INSERT INTO `s_core_log` (`id`, `type`, `key`, `text`, `date`, `user`, `ip_address`, `user_agent`, `value4`) VALUES(?,?,?,?,?,?,?,?,?)';

      $prms_id         = NULL;
      $prms_type       = 'backend';
      $prms_key        = 'Plugin - '.self::getLabel();
      $prms_text       = $msg;
      $prms_date       = date('Y-m-d H:i:s');
      $prms_user       = self::getLabel();
      $prms_ip_address = '';
      $prms_user_agent = '';
      $prms_value4     = '';

      $params = array($prms_id, $prms_type, $prms_key, $prms_text, $prms_date, $prms_user, $prms_ip_address, $prms_user_agent, $prms_value4);

      return Shopware()->Db()->query($sql, $params);

    } catch (Exception $e) {
      self::Logging('Logging | '.$e->getMessage());
      return;
    }
  }

}