<?php

namespace MicropaymentOFApi\Subscriber;

use Enlight\Event\SubscriberInterface;
use Shopware\Components\Plugin\ConfigReader;
use Shopware_Controllers_Frontend_MicropaymentSepa;
use Shopware_Controllers_Frontend_MicropaymentPrepay;
use Shopware_Controllers_Frontend_MicropaymentCcard;

class RouteSubscriber implements SubscriberInterface
{
  private $pluginDirectory;
  private $config;

  public static function getSubscribedEvents()
  {
    return [
      //'Enlight_Controller_Action_PostDispatchSecure_Frontend'            => 'onPostDispatch',
      'Enlight_Controller_Action_PreDispatch_Frontend_Checkout'          => 'onPreDispatch',
      //'Enlight_Controller_Action_PreDispatch_Frontend_MicropaymentCcard' => 'onPreDispatch',
      //'Enlight_Controller_Action_PreDispatch_Frontend_MicropaymentSepa'  => 'onPreDispatch',
      'Enlight_Controller_Action_PreDispatch_Frontend_MicropaymentGateway'  => 'onPreDispatch',
      //'Enlight_Controller_Action_PreDispatch_Frontend_MicropaymentPrepay' => 'onPreDispatch',
    ];
  }

  public function __construct($pluginName, $pluginDirectory, ConfigReader $configReader)
  {
    $this->pluginDirectory = $pluginDirectory;
  }

  /**
   * onPreDispatch
   *
   * @param \Enlight_Controller_ActionEventArgs $args
   */
  public function onPreDispatch(\Enlight_Controller_ActionEventArgs $args)
  {
    /** @var \Enlight_Controller_Action $controller */
    $controller = $args->get('subject');
    $view       = $controller->View();


    $view->addTemplateDir($this->pluginDirectory.'/Resources/views');

    $action = $args->getRequest()->getActionName();
    if (in_array($action, array('getStyle', 'getScript', 'getImage'))) {
      Shopware()->Plugins()->Controller()->ViewRenderer()->setNoRender();
    }

    // show prepay text on finish page
    if (in_array($action, array('finish'))){
      if (!empty(Shopware()->Session()->McPayAPIChkSum)) {
        require_once(__DIR__.'/../mcpay/classes/class.mcpay.php');
        $view->assign('mcpayapiimg',\mcpay::getPoweredBy(FALSE, Shopware()->Session()->McPayAPIChkSum, 9));
        Shopware()->Session()->McPayAPIChkSum = FALSE; // no reuse for chksum
      }
      // prepayment infos text
      if (!empty(Shopware()->Session()->McPayAPIPrepayText)) {
        $view->assign('mcpayapi_infos', Shopware()->Session()->McPayAPIPrepayText);
        Shopware()->Session()->McPayAPIPrepayText = false; // no need to save this longer
      }
      // sepa info text
      if (!empty(Shopware()->Session()->McPayAPIMandatText)){
        $view->assign('mcpayapi_infos', Shopware()->Session()->McPayAPIMandatText);
        Shopware()->Session()->McPayAPIMandatText = false; // no need to save this longer
      }
    }

    if (in_array($action, array('shippingPayment'))) {
      $req = $args->getRequest()->getParams();
      //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$req', print_r($req, 1));
    }

    if (in_array($action, array('saveShippingPayment'))) {
      $req = $args->getRequest()->getParams();
      if (!empty($req['isXHR'])){
        // change of payment selection
      } else {
        // filter payment input fields
        $fields = array();
        foreach ($req AS $k => $v) {
          if (strpos($k, 'mcpay_') !== FALSE) {
            $fields[$k] = $v;
          }
        }

        $paymentID = $req['payment'];
        $pm        = $this->getPaymentmean($paymentID);
        //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$req', print_r($req, 1).print_r($fields,1));

        if ($pm['name'] == 'mcpayapi_sepaXX') {
          require_once(__DIR__.'/../Controllers/Frontend/MicropaymentSepa.php');
          // save data
          if (!empty($req['mcpay_sepa_remember']) && ($req['mcpay_sepa_reuse'] == 'new' || !isset($req['mcpay_sepa_reuse']))) {
            $fields['pmid'] = $paymentID;
            Shopware_Controllers_Frontend_MicropaymentSepa::saveFields($fields, $paymentID);
          }
          if (!empty($req['mcpay_sepa_reuse']) && $req['mcpay_sepa_reuse'] != 'new'){
            $fields = Shopware_Controllers_Frontend_MicropaymentSepa::loadFields($paymentID);
            $fields['pmid'] = $paymentID;
            //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$fields', print_r($fields,1));
          } else {
            $options = Shopware_Controllers_Frontend_MicropaymentSepa::checkFields($req);
            // display errors
            if (!empty($options)) {
              Shopware()->Session()->mcpay_options = $options;
              $args->getSubject()->redirect(
                array(
                  'controller'  => 'checkout',
                  'action'      => 'shippingPayment',
                  'forceSecure' => 1,
                  'sTarget'     => 'checkout',
                )
              );
            }
          }
        } else if ($pm['name'] == 'mcpayapi_ccardXX') {
          require_once(__DIR__.'/../Controllers/Frontend/MicropaymentCcard.php');
          // save data
          if (!empty($req['mcpay_card_remember']) && ($req['mcpay_card_reuse'] == 'new' || !isset($req['mcpay_card_reuse']))) {
            $fields['pmid'] = $paymentID;
            $fields['reuse'] = false;
            $fields['mcpay_card_reuse'] = 'new';
            Shopware_Controllers_Frontend_MicropaymentCcard::saveFields($fields, $paymentID);
          } else {
            if ((empty($req['mcpay_card_reuse']) || $req['mcpay_card_reuse'] == 'new') && empty($req['mcpay_card_remember'])){
              // remove saved creditcard while using new card
              Shopware_Controllers_Frontend_MicropaymentCcard::deleteFields($paymentID);
            }
          }
          if (!empty($req['mcpay_card_reuse']) && $req['mcpay_card_reuse'] != 'new'){
            $fields = Shopware_Controllers_Frontend_MicropaymentCcard::loadFields($paymentID);
            $fields['reuse'] = true;
            $fields['pmid'] = $paymentID;
            //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$fields', print_r($fields,1));
          } else {
            $options = Shopware_Controllers_Frontend_MicropaymentCcard::checkFields($req);
            // display errors
            if (!empty($options)) {
              //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$options', print_r($options,1));
              Shopware()->Session()->mcpay_options = $options;
              $args->getSubject()->redirect(
                array(
                  'controller'  => 'checkout',
                  'action'      => 'shippingPayment',
                  'forceSecure' => 1,
                  'sTarget'     => 'checkout',
                )
              );
            }
          }
        } else if ($pm['name'] == 'mcpayapi_prepay') {

        } else {
          //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->Shopware()->Session()->mcpay_options', print_r(Shopware()->Session()->mcpay_options,1));
          Shopware()->Session()->mcpay_options = array(); // clean errors
        }

        // save fields in session
        //mail('webmaster@web-dezign.de', __CLASS__.'->'.__FUNCTION__.'->$fields', print_r($fields,1));
        Shopware()->Session()->mcpay_fields = $fields;
      }
    }
  }

  /**
   * onPostDispatch
   *
   * @param \Enlight_Controller_ActionEventArgs $args
   */
//  public function onPostDispatch(\Enlight_Controller_ActionEventArgs $args)
//  {
//    /** @var \Enlight_Controller_Action $controller */
//    $controller = $args->get('subject');
//    $view       = $controller->View();
//    $view->addTemplateDir($this->pluginDirectory.'/Resources/views');
//  }

  /**
   * getPaymentmean
   *
   * @param $paymentId
   *
   * @return mixed
   */
  public function getPaymentmean($paymentId)
  {
    $sql  = 'SELECT *
            FROM `s_core_paymentmeans`
            WHERE `id` = "'.$paymentId.'"
            ';
    $data = current(Shopware()->Db()->fetchAll($sql));
    return $data;
  }
}